# copyright (c) 2008 Dmitry Dianov. All rights reserved.

from trac.core import Component, implements
from trac.web.api import ITemplateStreamFilter
from genshi.builder import tag
from genshi.core import Markup
from genshi.filters.transform import Transformer
from trac.web.chrome import ITemplateProvider, add_stylesheet, add_script
from pkg_resources import resource_filename
from trac.web import IRequestFilter
from tractags.api import TagSystem
import re

class KeywordSuggestModule(Component):
    implements (ITemplateStreamFilter, ITemplateProvider, IRequestFilter)

    # ITemplateStreamFilter
    def filter_stream(self, req, method, filename, stream, data):
        if (filename <> 'ticket.html' and filename <> 'wiki_edit.html'):
            return stream

# 2008-12-03 modified by george to get keywords from tag system then from suggest list in trac.ini

        keywords = self.config.getlist('keywordsuggest','keywords')
# merge with tags
        query_result = TagSystem(self.env).query(req, '')
        for resource, tags in query_result:
            for _tag in tags:
                if (_tag not in keywords):
                    keywords.append(_tag)

        if keywords:
            keywords = ','.join(("'%s'" % keyword for keyword in keywords))
        else:
            keywords = ''
        
#        self.log.debug("[george is debugging] %s" % keywords)

        if keywords == '':
            self.log.debug('List of keywords not found in trac.ini. and no tags found'\
                           'Plugin keywordsuggest disabled.')
            return stream

        mustmatch = 'mustMatch: true,'
        if not self.config.getbool('keywordsuggest','mustmatch'):
            mustmatch = ""
        sep = self.config.get('keywordsuggest','multipleseparator', '')[1:-1] or ', '
        matchcontains = 'matchContains: true,'
        if not self.config.getbool('keywordsuggest','matchcontains',True):
            matchcontains = ""

#        self.log.debug('[george is debugging] path %s' % req.path_info)
       # inject transient part of javascript directly into ticket.html template
        if req.path_info.startswith('/ticket/') or \
           req.path_info.startswith('/newticket'):
#            self.log.debug('[george is debugging] javascript for ticket before loading')
            js_ticket = """
            $(function($) {
              var sep = '%s'
              $('#field-keywords').autocomplete([%s], {multiple: true, %s
                                                %s multipleSeparator: sep, autoFill: true}); 
              $("form").submit(function() {
                  // remove trail separator if any
                  keywords = $("input#field-keywords").attr('value')
                  if (keywords.lastIndexOf(sep) + sep.length == keywords.length) {
                      $("input#field-keywords").attr('value', keywords.substring(0, keywords.length-sep.length))
                  }
              });
            });""" % (sep, keywords, matchcontains, mustmatch)
            stream = stream | Transformer('.//head').append \
                              (tag.script(Markup(js_ticket), type='text/javascript'))

            # turn keywords field label into link to wiki page
            helppage = self.config.get('keywordsuggest','helppage')
            if helppage:
                link = tag.a(href='wiki/%s' % helppage, target='blank')
                if not self.config.getbool('keywordsuggest','helppage.newwindow',
                                           'false'):
                    link.attrib -= 'target'
                stream = stream | Transformer\
                             ('//label[@for="field-keywords"]/text()').wrap(link)

        elif req.path_info.startswith('/wiki/'):
#            self.log.debug('[george is debugging] javascript for wiki before loading')
            js_wiki = """
            $(function($) {
              var sep = '%s'
              $('#tags').autocomplete([%s], {multiple: true, %s
                                                %s multipleSeparator: sep, autoFill: true}); 
              $("form").submit(function() {
                  // remove trail separator if any
                  keywords = $("input#tags").attr('value')
                  if (keywords.lastIndexOf(sep) + sep.length == keywords.length) {
                      $("input#tags").attr('value', keywords.substring(0, keywords.length-sep.length))
                  }
              });
            });""" % (sep, keywords, matchcontains, mustmatch)
            stream = stream | Transformer('.//head').append \
                              (tag.script(Markup(js_wiki), type='text/javascript'))
#            self.log.debug('[george is debugging] javascript for wiki after loading')

        return stream

    # ITemplateProvider methods
    def get_htdocs_dirs(self):
        """Return the absolute path of a directory containing additional
        static resources (such as images, style sheets, etc).
        """
        return [('keywordsuggest', resource_filename(__name__, 'htdocs'))]

    def get_templates_dirs(self):
        return [resource_filename(__name__, 'htdocs')]

    # IRequestFilter methods
    def pre_process_request(self, req, handler):
        return handler
    
    def post_process_request(self, req, template, data, content_type):
        if req.path_info.startswith('/ticket/') or \
           req.path_info.startswith('/newticket') or \
           req.path_info.startswith('/wiki/'):
            add_script(req, 'keywordsuggest/jquery.bgiframe.min.js')
            add_script(req, 'keywordsuggest/jquery.autocomplete.pack.js')
            add_stylesheet(req, 'keywordsuggest/autocomplete.css')
        return template, data, content_type
